using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Linq;
using MVR;
using MeshVR;
using MVR.FileManagementSecure;
using Unity.Collections;
using Battlehub.RTSaveLoad.UnityEngineNS.AINS;

namespace Stopper {
	public class LinkTriggers : MVRScript {

        List<FreeControllerV3> controllers;
        FreeControllerV3 mainController;

		public override void Init() {
            controllers = this.containingAtom.freeControllers.Where(fc => fc.isActiveAndEnabled).ToList();
            mainController = this.containingAtom.mainController;

            foreach(FreeControllerV3 controller in controllers) {
                try {
                    JSONStorableStringChooser linkToRigidbody = new JSONStorableStringChooser(
                        "linkToRigidbody", 
                        new List<string>(){"None"},
                        "None", 
                        "Link To Rigidbody", 
                        (string uid) => {
                            controller.SetLinkToRigidbodyObject(uid);
                        });
                    linkToRigidbody.isStorable = false;

                    JSONStorableStringChooser linkAtomChooser = new JSONStorableStringChooser(
                        "linkToAtom", 
                        GetAtomUids(),
                        "None", 
                        "Link To Atom", 
                        (string uid) => {
                            controller.linkToAtomSelectionPopup.currentValue = uid;
                            if(uid == "None") {
                                linkToRigidbody.choices = new List<string>(){"None"};
                            } else {
                                linkToRigidbody.choices = GetAtomRigidbodies(uid);
                            }
                        });
                    linkAtomChooser.isStorable = false;

                    controller.RegisterStringChooser(linkAtomChooser);
                    controller.RegisterStringChooser(linkToRigidbody);
                } catch (Exception e) {
                    SuperController.LogError($"Could not add link triggers to {controller}. {e}");
                }
            }

            

            JSONStorableStringChooser parentAtomChooser = new JSONStorableStringChooser(
					"parentAtom", 
					GetAtomUids(),
					"None", 
					"Parent Atom", 
					(string uid) => {
                        this.containingAtom.parentAtomSelectionPopup.currentValue = uid;
					});
            mainController.RegisterStringChooser(parentAtomChooser);

            SuperController.singleton.onAtomAddedHandlers += RefreshAtoms;
            SuperController.singleton.onAtomRemovedHandlers += RefreshAtoms;
            SuperController.singleton.onAtomUIDsChangedHandlers += RefreshAtoms;
		}

        private List<string> GetAtomUids() {
            List<string> atomUids = new List<string>(){"None"};
            atomUids.AddRange(SuperController.singleton.GetAtoms().Select(a => a.uid));
            return atomUids;
        }

        private List<string> GetAtomRigidbodies(string atomUid) {
            List<string> rbs = new List<string>(){"None"};
            rbs.AddRange(SuperController.singleton.GetRigidbodyNamesInAtom(atomUid));
            return rbs;
        }

        private void RefreshAtoms(object unused) {
            foreach(FreeControllerV3 controller in controllers) {
                JSONStorableStringChooser linkAtomChooser = controller.GetStringChooserJSONParam("linkToAtom");
                if(linkAtomChooser != null) {
                    linkAtomChooser.choices = GetAtomUids();
                }
            }

            JSONStorableStringChooser parentAtomChooser = mainController.GetStringChooserJSONParam("parentAtom");
                if(parentAtomChooser != null) {
                    parentAtomChooser.choices = GetAtomUids();
                }
        }

        public void OnDestroy() {
            foreach(FreeControllerV3 controller in controllers) {
                JSONStorableStringChooser linkAtomChooser = controller.GetStringChooserJSONParam("linkToAtom");
                if(linkAtomChooser != null) {
                    controller.DeregisterStringChooser(linkAtomChooser);
                }

                JSONStorableStringChooser linkToRigidbody = controller.GetStringChooserJSONParam("linkToRigidbody");
                if(linkToRigidbody != null) {
                    controller.DeregisterStringChooser(linkToRigidbody);
                }
            }

            if(mainController != null) {
                JSONStorableStringChooser parentAtomChooser = mainController.GetStringChooserJSONParam("parentAtom");
                if(parentAtomChooser != null) {
                    mainController.DeregisterStringChooser(parentAtomChooser);
                }
            }

            SuperController.singleton.onAtomAddedHandlers -= RefreshAtoms;
            SuperController.singleton.onAtomRemovedHandlers -= RefreshAtoms;
            SuperController.singleton.onAtomUIDsChangedHandlers -= RefreshAtoms;
        }
	}
}